/*
 * gps.h -- The user accessible API for libgps.  
 *
 *  Copyright 1997 John E. Stone (j.stone@acm.org)
 *
 *  $Id: gps.h,v 1.1 2007/09/02 06:42:04 johns Exp $
 */

#ifdef  __cplusplus
extern "C" {
#endif


/* bitwise ORed flags specifying what data we've received so far */
#define GPS_COMM    0x01  /* we've received some valid NMEA packets   */
#define GPS_FIXLAT  0x02  /* GPS has a latitude fix                   */
#define GPS_FIXLONG 0x04  /* GPS has a longtitude fix                 */
#define GPS_FIXQ    0x08  /* GPS has reported fix quality level       */
#define GPS_DOP     0x10  /* GPS has reported dilution of precision   */
#define GPS_SATS    0x20  /* GPS has reported number of visiable sats */
#define GPS_TIME    0x40  /* GPS has reported the time of day (UTC)   */
#define GPS_DATE    0x80  /* GPS has reported the date (UTC)          */

/* convenience macros */
#define GPS_TIMELOC (GPS_COMM | GPS_DATE | GPS_TIME | GPS_FIXLAT | GPS_FIXLONG)


typedef void * GPSHandle; /* GPS Handle type, used by all gps API functions */

/* 
 * gps_open() 
 *   Open a named serial port which a GPS is attached to.
 * Returns a handle which is used by all other gps API functions.
 * If the serial port open fails, or the gps does not pass initialization
 * tests, then a NULL is returned as the handle.   
 */
GPSHandle gps_open(const char * gpsname); 

/*
 * gps_close()
 *   Closes down the GPS serial port, frees allocated resources and
 * discards any unprocessed gps messages.
 */ 
int gps_close(GPSHandle voidhandle);

/*
 * gps_getstatus()
 *   Polls the GPS serial port for new packets, performs any optional
 * postprocessing of GPS data.
 */
int gps_clear_data(GPSHandle voidhandle);
int gps_have_data(GPSHandle voidhandle, int *havedata);
int gps_get_status(GPSHandle voidhandle);
int gps_get_fixquality(GPSHandle voidhandle, int *fixq);
int gps_get_numsats(GPSHandle voidhandle, int *sats);
int gps_get_dop(GPSHandle voidhandle, float *dop);
int gps_get_date_utc(GPSHandle voidhandle, int *day, int *month, int *year);
int gps_get_time_utc(GPSHandle voidhandle, int *hour, int *mins, int *secs);
int gps_sprint_date_utc(GPSHandle voidhandle, char *buf);
int gps_sprint_time_utc(GPSHandle voidhandle, char *buf);
int gps_get_lat_decimal(GPSHandle voidhandle, float *lat);
int gps_get_long_decimal(GPSHandle voidhandle, float *lon);
int gps_sprint_lat(GPSHandle voidhandle, char *buf);
int gps_sprint_long(GPSHandle voidhandle, char *buf);
int gps_systime_diff(GPSHandle voidhandle, int *diffsecs);

#ifdef  __cplusplus
}
#endif


