/*
 * rs232serial.h -- Machine/OS dependent serial I/O code for Librs232
 *
 *  Copyright 1997 John E. Stone (j.stone@acm.org)
 *
 *  $Id: serial.h,v 1.1 2007/09/02 06:42:04 johns Exp $
 *
 */

/*
 * Machine/OS dependent serial port I/O routines.
 *
 * rs232_comm_open() - open the serial port device for communication with
 *                   the rs232.  Settings are 9600,N,8,1, non-blocking,
 *                   no controlling tty.
 * rs232_comm_read() - nonblocking read of up to size bytes
 * rs232_comm_write() - blocking write of up to size bytes
 * rs232_comm_close() - close the serial port device
 */

typedef void * RS232CommHandle;

int rs232_comm_open(const char * commname, RS232CommHandle * commhandleptr, int baud); 
int rs232_comm_write(RS232CommHandle commhandle, const char * buf); 
int rs232_comm_read(RS232CommHandle commhandle, char * buf, int sz); 
int rs232_comm_close(RS232CommHandle * commhandleptr); 
int rs232_comm_get_status(RS232CommHandle, int *dsr, int *dtr, int *rts, int *cts, int *cd, int *rng);
int rs232_comm_set_rts(RS232CommHandle, int level);
int rs232_comm_set_dtr(RS232CommHandle, int level);
