
#define SOCKINTERNAL 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(_MSC_VER) 
#include <winsock2.h>
#else
#include <arpa/inet.h>
#include <fcntl.h>

#include <sys/types.h>
#include <unistd.h>   /* for Linux */
#include <sys/socket.h>
#include <netdb.h>
#endif

#include <errno.h>

#include "sock.h"

#if !defined(INADDR_NONE)
#define INADDR_NONE -1
#endif

int sock_init(void) {
#if defined(_MSC_VER)
  int rc = 0;
  static int initialized=0;

  if (!initialized) {
    WSADATA wsdata;
    rc = WSAStartup(MAKEWORD(1,1), &wsdata);
    if (rc == 0)
      initialized = 1;
  }

  return rc;
#else   
  return 0;
#endif
}


void * sock_create(void) {
  csocket * s;

  s = (csocket *) malloc(sizeof(csocket));
  if (s != NULL)
    memset(s, 0, sizeof(csocket)); 

  if ((s->sd = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
    printf("Failed to open socket.");
    free(s);
    return NULL;
  }

  return (void *) s;
}

int  sock_connect(void *v, const char *host, int port) {
  csocket *s = (csocket *) v;
  char address[1030];
  struct hostent *h;

  memset(&(s->addr), 0, sizeof(s->addr)); 
  s->addr.sin_family = PF_INET;

  h=gethostbyname(host);
  if (h != NULL) {
    sprintf(address, "%d.%d.%d.%d",
      (unsigned char) h->h_addr_list[0][0],
      (unsigned char) h->h_addr_list[0][1],
      (unsigned char) h->h_addr_list[0][2],
      (unsigned char) h->h_addr_list[0][3]);
#if 0
    if (!inet_aton(address, &s->addr.sin_addr))
      return -1;
#else
    if ((s->addr.sin_addr.s_addr = inet_addr(address)) == INADDR_NONE)
      return -1;
#endif
  } else {
    /* failed to lookup address, try parsing as an IP address */
#if 0
    if (!inet_aton(address, &s->addr.sin_addr))
      return -1;
#else
    if ((s->addr.sin_addr.s_addr = inet_addr(address)) == INADDR_NONE)
      return -1;
#endif
  }

#if 1
#else
#endif

  s->addr.sin_port = htons(port);  

  return connect(s->sd, (struct sockaddr *) &s->addr, sizeof(s->addr)); 
}

int sock_bind(void * v, int port) {
  csocket *s = (csocket *) v;
  memset(&(s->addr), 0, sizeof(s->addr)); 
  s->addr.sin_family = PF_INET;
  s->addr.sin_port = htons(port);

  return bind(s->sd, (struct sockaddr *) &s->addr, sizeof(s->addr));
}

int sock_listen(void * v) {
  csocket *s = (csocket *) v;
  return listen(s->sd, 5);
}

void *sock_accept(void * v) {
  int rc;
  csocket *new_s = NULL, *s = (csocket *) v;
#if defined(__aix)
  unsigned int len;
#elif defined(SOCKLEN_T)
  SOCKLEN_T len;
#elif defined(__linux__)
  socklen_t len;
#else
  int len;
#endif

  len = sizeof(s->addr);
  rc = accept(s->sd, (struct sockaddr *) &s->addr, &len);
  if (rc >= 0) {
    new_s = (csocket *) malloc(sizeof(csocket));
    if (new_s != NULL) {
      *new_s = *s;
      new_s->sd = rc;
    }
  }
  return (void *)new_s;
}

int  sock_write(void * v, const void *buf, int len) {
  csocket *s = (csocket *) v;
#if defined(_MSC_VER)
  return send(s->sd, (const char*) buf, len, 0);  // windows lacks the write() call
#else
  return write(s->sd, buf, len);
#endif
}

int  sock_read(void * v, void *buf, int len) {
  csocket *s = (csocket *) v;
#if defined(_MSC_VER)
  return recv(s->sd, (char*) buf, len, 0); // windows lacks the read() call
#else
  return read(s->sd, buf, len);
#endif

}

void sock_shutdown(void *v) {
  csocket * s = (csocket *) v;
  if (s == NULL)
    return;

#if defined(_MSC_VER)
  shutdown(s->sd, SD_SEND);
#else
  shutdown(s->sd, 1);  /* complete sends and send FIN */
#endif
}

void sock_destroy(void * v) {
  csocket * s = (csocket *) v;
  if (s == NULL)
    return;

#if defined(_MSC_VER)
  closesocket(s->sd);
#else
  close(s->sd);
#endif
  free(s);  
}

int sock_selread(void *v, int sec) {
  csocket *s = (csocket *)v;
  fd_set rfd;
  struct timeval tv;
  int rc;
 
  FD_ZERO(&rfd);
  FD_SET(s->sd, &rfd);
  memset((void *)&tv, 0, sizeof(struct timeval));
  tv.tv_sec = sec;
  do {
    rc = select(s->sd+1, &rfd, NULL, NULL, &tv);
  } while (rc < 0 && errno == EINTR);
  return rc;

}
  
int sock_selwrite(void *v, int sec) {
  csocket *s = (csocket *)v;
  fd_set wfd;
  struct timeval tv;
  int rc;
 
  FD_ZERO(&wfd);
  FD_SET(s->sd, &wfd);
  memset((void *)&tv, 0, sizeof(struct timeval));
  tv.tv_sec = sec;
  do {
    rc = select(s->sd + 1, NULL, &wfd, NULL, &tv);
  } while (rc < 0 && errno == EINTR);
  return rc;
}
